#if !defined(__FLLOPQPS_H)
#define	__FLLOPQPS_H
#include "fllopqp.h"
        
typedef struct _p_QPS* QPS;

#define QPSType char*
#define QPSKSP          "ksp"
#define QPSMPGP         "mpgp"
#define QPSSMALSE       "smalse"

FLLOP_EXTERN PetscClassId QPS_CLASSID;

FLLOP_EXTERN PetscErrorCode QPSInitializePackage(const char []);

FLLOP_EXTERN PetscErrorCode QPSCreate(MPI_Comm comm,QPS *qps);
FLLOP_EXTERN PetscErrorCode QPSView(QPS qps,PetscViewer v);
FLLOP_EXTERN PetscErrorCode QPSDestroy(QPS *qps);
FLLOP_EXTERN PetscErrorCode QPSSetFromOptions(QPS qps);
FLLOP_EXTERN PetscErrorCode QPSSetUp(QPS qps);
FLLOP_EXTERN PetscErrorCode QPSReset(QPS qps);
FLLOP_EXTERN PetscErrorCode QPSSolve(QPS qps);
FLLOP_EXTERN PetscErrorCode QPSPostSolve(QPS qps);
FLLOP_EXTERN PetscErrorCode QPSIsQPCompatible(QPS qps,QP qp,PetscBool *flg);

FLLOP_EXTERN PetscErrorCode QPSSetDefaultType(QPS qps);
FLLOP_EXTERN PetscErrorCode QPSSetType(QPS qps,const QPSType type);
FLLOP_EXTERN PetscErrorCode QPSSetQP(QPS qps,QP qp);
FLLOP_EXTERN PetscErrorCode QPSSetTolerances(QPS qps,PetscReal rtol,PetscReal abstol,PetscReal dtol,PetscInt maxits);
FLLOP_EXTERN PetscErrorCode QPSSetOptionsPrefix(QPS qps,const char prefix[]);
FLLOP_EXTERN PetscErrorCode QPSAppendOptionsPrefix(QPS qps,const char prefix[]);
FLLOP_EXTERN PetscErrorCode QPSSetConvergenceTest(QPS qps,PetscErrorCode (*converge)(QPS),void *cctx,PetscErrorCode (*destroy)(void*));
FLLOP_EXTERN PetscErrorCode QPSSetAutoPostSolve(QPS qps,PetscBool flg);
FLLOP_EXTERN PetscErrorCode QPSSetUsePC(QPS qps,PetscBool flg);

FLLOP_EXTERN PetscErrorCode QPSGetType(QPS qps,const QPSType *type);
FLLOP_EXTERN PetscErrorCode QPSGetQP(QPS qps,QP *qp);
FLLOP_EXTERN PetscErrorCode QPSGetSolvedQP(QPS qps,QP *qp);
FLLOP_EXTERN PetscErrorCode QPSGetPC(QPS qps,PC *pc);
FLLOP_EXTERN PetscErrorCode QPSGetTolerances(QPS qps,PetscReal *rtol,PetscReal *abstol,PetscReal *dtol,PetscInt *maxits);
FLLOP_EXTERN PetscErrorCode QPSGetOptionsPrefix(QPS qps,const char *prefix[]);
FLLOP_EXTERN PetscErrorCode QPSGetConvergenceContext(QPS,void **);
FLLOP_EXTERN PetscErrorCode QPSGetConvergedReason(QPS,KSPConvergedReason *);
FLLOP_EXTERN PetscErrorCode QPSGetResidualNorm(QPS qps,PetscReal *rnorm);
FLLOP_EXTERN PetscErrorCode QPSGetIterationNumber(QPS qps,PetscInt *its);
FLLOP_EXTERN PetscErrorCode QPSGetAutoPostSolve(QPS qps,PetscBool *flg);
FLLOP_EXTERN PetscErrorCode QPSGetUsePC(QPS qps,PetscBool *flg);

FLLOP_EXTERN PetscFList QPSList;
FLLOP_EXTERN PetscBool  QPSRegisterAllCalled;
FLLOP_EXTERN PetscErrorCode QPSRegisterAll(const char[]);
FLLOP_EXTERN PetscErrorCode QPSRegisterDestroy(void);
FLLOP_EXTERN PetscErrorCode QPSRegister(const char[],const char[],const char[],PetscErrorCode (*)(QPS));
#if defined(PETSC_USE_DYNAMIC_LIBRARIES)
#define QPSRegisterDynamic(a,b,c,d) QPSRegister(a,b,c,0)
#else
#define QPSRegisterDynamic(a,b,c,d) QPSRegister(a,b,c,d)
#endif

FLLOP_EXTERN PetscErrorCode QPSDefaultConverged(QPS);
FLLOP_EXTERN PetscErrorCode QPSDefaultConvergedDestroy(void *);
FLLOP_EXTERN PetscErrorCode QPSDefaultConvergedCreate(void **);
FLLOP_EXTERN PetscErrorCode QPSSkipConverged(QPS);


/* *** type-specific stuff *** */
/* KSP */
FLLOP_EXTERN PetscErrorCode QPSKSPSetKSP(QPS qps,KSP ksp);
FLLOP_EXTERN PetscErrorCode QPSKSPGetKSP(QPS qps,KSP *ksp);
FLLOP_EXTERN PetscErrorCode QPSKSPSetType(QPS qps,const KSPType type);
FLLOP_EXTERN PetscErrorCode QPSKSPGetType(QPS qps,const KSPType *type);

/* MPGP */
FLLOP_EXTERN PetscErrorCode QPSMPGPSetGamma(QPS qps,PetscReal gamma);
FLLOP_EXTERN PetscErrorCode QPSMPGPGetGamma(QPS qps,PetscReal *gamma);

#endif

