#if !defined(__FLLOPSYS_H)
#define	__FLLOPSYS_H

#include <petscsys.h>
#include <petscksp.h>
#include <petscmath.h>

#if defined(PETSC_HAVE_TIME_H)
#include <time.h>
#endif
#if defined(PETSC_HAVE_SYS_TIME_H)
#include <sys/time.h>
#endif
#if defined(PETSC_HAVE_SYS_TYPES_H)
#include <sys/types.h>
#include <sys/stat.h>
#endif

#define FLLOP_SETERRQ( comm,n,s) SETERRQ(comm,n,s)
#define FLLOP_SETERRQ1(comm,n,s,a1) SETERRQ1(comm,n,s,a1)
#define FLLOP_SETERRQ2(comm,n,s,a1,a2) SETERRQ2(comm,n,s,a1,a2)
#define FLLOP_SETERRQ3(comm,n,s,a1,a2,a3) SETERRQ3(comm,n,s,a1,a2,a3)
#define FLLOP_SETERRQ4(comm,n,s,a1,a2,a3,a4) SETERRQ4(comm,n,s,a1,a2,a3,a4)
#define FLLOP_SETERRQ5(comm,n,s,a1,a2,a3,a4,a5) SETERRQ5(comm,n,s,a1,a2,a3,a4,a5)

#define FLLOP_EXTERN PETSC_EXTERN

#include "flloppetscretro.h"

#ifdef NAME_MAX
#define MAX_NAME_LEN NAME_MAX
#else
#define MAX_NAME_LEN 255
#endif

#define PATH_BUFFER_LEN  PETSC_MAX_PATH_LEN
FLLOP_EXTERN char FLLOP_PathBuffer_Global[PATH_BUFFER_LEN];

#define OBJ_NAME_BUFFER_LEN   MAX_NAME_LEN
FLLOP_EXTERN char FLLOP_ObjNameBuffer_Global[OBJ_NAME_BUFFER_LEN];

/* BEGIN Function-like Macros */
FLLOP_EXTERN PetscErrorCode _fllop_ierr;
#define TRY(f) do {_fllop_ierr = f; CHKERRQ(_fllop_ierr);} while (0)
#define FLLOP_SETERRQ_WORLD( n,s) FLLOP_SETERRQ(PETSC_COMM_WORLD,n,s)
#define FLLOP_SETERRQ_WORLD1(n,s,a1) FLLOP_SETERRQ1(PETSC_COMM_WORLD,n,s,a1)
#define FLLOP_SETERRQ_WORLD2(n,s,a1,a2) FLLOP_SETERRQ2(PETSC_COMM_WORLD,n,s,a1,a2)
#define FLLOP_SETERRQ_WORLD3(n,s,a1,a2,a3) FLLOP_SETERRQ3(PETSC_COMM_WORLD,n,s,a1,a2,a3)
#define FLLOP_SETERRQ_WORLD4(n,s,a1,a2,a3,a4) FLLOP_SETERRQ4(PETSC_COMM_WORLD,n,s,a1,a2,a3,a4)
#define FLLOP_SETERRQ_WORLD5(n,s,a1,a2,a3,a4,a5) FLLOP_SETERRQ5(PETSC_COMM_WORLD,n,s,a1,a2,a3,a4,a5)
#define FLLOP_ASSERT(cond, condStr) if(!(cond)) FLLOP_SETERRQ1(PETSC_COMM_WORLD, -1, "Assertion failed: %s", (condStr));
#define PetscIntGlobalSum( comm, loc, glob) MPI_Allreduce((PetscMPIInt*) loc, (PetscMPIInt*) glob, 1, MPIU_INT,    MPIU_SUM, comm)
#define PetscIntGlobalMax( comm, loc, glob) MPI_Allreduce((PetscMPIInt*) loc, (PetscMPIInt*) glob, 1, MPIU_INT,    MPIU_MAX, comm)
#define PetscIntGlobalMin( comm, loc, glob) MPI_Allreduce((PetscMPIInt*) loc, (PetscMPIInt*) glob, 1, MPIU_INT,    MPIU_MIN, comm)
#define PetscBoolGlobalAnd(comm, loc, glob) MPI_Allreduce((PetscMPIInt*) loc, (PetscMPIInt*) glob, 1, MPI_LOGICAL, MPI_LAND, comm)
#define PetscBoolGlobalOr( comm, loc, glob) MPI_Allreduce((PetscMPIInt*) loc, (PetscMPIInt*) glob, 1, MPI_LOGICAL, MPI_LOR,  comm)

FLLOP_EXTERN PetscBool fllop_trace;
FLLOP_EXTERN PetscInt PeFuBe_i_;
FLLOP_EXTERN char     PeFuBe_s_[128];

#define PetscFunctionBeginI \
PetscLogDouble ttttt;\
if (fllop_trace) {\
    PetscFunctionBegin;\
    PeFuBe_s_[PeFuBe_i_]=' ';\
    PeFuBe_s_[PeFuBe_i_+1]=0;\
    TIC(&ttttt);\
    TRY( PetscPrintf(PETSC_COMM_WORLD,"%s%d BEGIN FUNCTION %s\n",PeFuBe_s_,PeFuBe_i_,__FUNCT__) );\
    PeFuBe_i_++;\
} else {\
    PetscFunctionBegin;\
}

#define PetscFunctionReturnI(rrrrr) \
{\
if (fllop_trace) {\
    TOC(&ttttt);\
    TRY( PetscPrintf(PETSC_COMM_WORLD,"%s%d END   FUNCTION %s (%2.2f s)\n",PeFuBe_s_,--PeFuBe_i_,__FUNCT__,ttttt) );\
    PeFuBe_s_[PeFuBe_i_]=0;\
    PetscFunctionReturn(rrrrr);\
} else {\
    PetscFunctionReturn(rrrrr);\
}\
}

/* END Function-like Macros */



/* BEGIN Inline Functions */
PETSC_STATIC_INLINE void TIC(PetscLogDouble *t) {
    PetscGetTime(t);
}


PETSC_STATIC_INLINE void TOC(PetscLogDouble *t) {
    PetscLogDouble toc_time;
    PetscGetTime(&toc_time);
    *t = toc_time - *t;
}


PETSC_STATIC_INLINE PetscInt PetscExtRound(PetscReal val) {
    PetscInt val_int;
    val_int = (PetscInt) val;
    return (val - val_int < .5) ? val_int : val_int+1;
}


PETSC_STATIC_INLINE PetscScalar PetscExtLog10Scalar(PetscScalar a) {
    return PetscLogScalar(a) / PetscLogScalar(10.0);
}

PETSC_STATIC_INLINE PetscErrorCode GlobMaxTimeToZero(MPI_Comm comm, PetscLogDouble *t) {
    PetscLogDouble ttt;
    PetscErrorCode ierr;
    ierr = MPI_Reduce(t, &ttt, 1, MPIU_PETSCLOGDOUBLE, MPIU_MAX, 0, comm); CHKERRQ(ierr);
    *t = ttt;
    return 0;
}
/* END Inline Functions */


/* BEGIN Function Headers */
FLLOP_EXTERN PetscErrorCode FllopInitialize(int *argc, char ***args, const char file[], const char help[]);
FLLOP_EXTERN PetscErrorCode FllopFinalize();
FLLOP_EXTERN PetscErrorCode UtilsMakePath(const char *dir, mode_t mode);
FLLOP_EXTERN PetscErrorCode PetscExtSupertypeCompare(PetscObject obj, const char type_name[], PetscBool *same);
FLLOP_EXTERN PetscErrorCode PetscExtGetTimestamp(char date[],size_t len);
FLLOP_EXTERN PetscErrorCode FllopTrace(PetscBool flg);
/* END Function Headers */

#endif
