#if !defined(__QPSIMPL_H)
#define	__QPSIMPL_H
#include <fllopqps.h>

typedef struct _QPSOps *QPSOps;

struct _QPSOps {
  PetscErrorCode (*solve)(QPS);
  PetscErrorCode (*setup)(QPS);
  PetscErrorCode (*destroy)(QPS);
  PetscErrorCode (*view)(QPS,PetscViewer);
  PetscErrorCode (*setfromoptions)(QPS);
  PetscErrorCode (*reset)(QPS);
  PetscErrorCode (*isqpcompatible)(QPS,QP,PetscBool *);
};

struct _p_QPS {
  PETSCHEADER(struct _QPSOps);
  
  /* properties */
  QP               topQP;
  PC               pc;
  PetscReal        rtol;
  PetscReal        atol;
  PetscReal        divtol;
  PetscInt         max_it;
  PetscBool        autoPostSolve;
  PetscBool        usePC;
  
  /* holder for misc stuff associated with a particular iterative solver */
  void             *data;
  
  /* work vectors */
  PetscInt         nwork;
  Vec              *work;
  
  /* convergence tests */  
  PetscErrorCode   (*convergencetest)(QPS);
  PetscErrorCode   (*convergencetestdestroy)(void*);
  void             *convergencetestctx;

  /* current state */
  PetscReal        rnorm;
  PetscInt         iteration;
  PetscBool        setupcalled;
  PetscBool        postsolvecalled;
  KSPConvergedReason reason;
};

typedef struct {
  PetscReal rnorm0,ttol;
} QPSDefaultConvergedCtx;

FLLOP_EXTERN PetscErrorCode QPSDefaultGetWork(QPS qps,PetscInt nw);

FLLOP_EXTERN PetscLogEvent QPS_Solve,QPS_Solve_solve,QPS_PostSolve;

#endif
