#ifndef CON_AWF1MAT_H
#define CON_AWF1MAT_H

#include "genfile.h"
  

/**
   This class defines model for water flow in soils.
*/

class con_awf1mat
{
 public:
  con_awf1mat();    //constructor
  ~con_awf1mat();   //destructor

 
  void matcond (matrix &d,long ri,long ci,long ipp);
  void matcap (double &cc,long ri,long ci,long ipp);
  
  void matcond1d (matrix &d,long ri,long ci,long ipp);
  void matcond2d (matrix &d,long ri,long ci,long ipp);
  void matcond3d (matrix &d,long ri,long ci,long ipp);

  void read(XFILE *in);

  void rhs_volume (matrix &d,long ri,long ci,long ipp);
  void rhs1d1 (matrix &d,long ri,long ci,long ipp);
  void rhs2d1 (matrix &d,long ri,long ci,long ipp);
  void rhs3d1 (matrix &d,long ri,long ci,long ipp);

  double get_sw(double pw);
  double get_dsw_dpw(double pw);
  double get_krw(double pw);
  double get_phi();
  double get_kintr();
  double get_alpha();
  double get_kw();
  double get_muw();

  double get_kww(double pw,long ipp);

  double get_capww(double pw);

  double get_fw1(double pw,long ipp);

  double get_othervalue(long compother,double pw, long ipp);
  void print_othervalue_name(FILE *out,long compother);
  void values_correction (vector &nv, long ipp);
  void water_pressure_check(double &pw,long ipp);
  void updateval (long ipp);
  void initval(long ipp);

  /// returns ordered dof names
  void give_dof_names(namevart *dofname, long ntm);

  double give_water_pressure(long ipp);
  double give_suction(long ipp);
  double give_saturation_degree(long ipp);

  /// marks required non-transport quantities
  void give_reqntq(long *antq);

 private:
  
  waterflowtype model_type;
  double alpha,phi0,ks;
  double kw,rhow,muw0;
  double kintr;
  double ssat,sirr,alfa,ksat,gamaw;
  double hp_min,htz,r;
  double delta,expn;
  double k0,e0,lam0,lam,m,sigma_m_bar;
  double kappa,sigma_m_eff_bar,sigma_m_tot_bar,sigma_m_eff_init;
};  

#endif
