#define EXTERN
#include "globalt.h"
#include "globmatt.h"

#include "stochdrivert.h"
#include "trfelinit.h"
#include "ctlinterface.h"
#include "npglvec.h"
#include "npsolvert.h"

// for debugging only
#define DEBUG
#define CTL_Connect
#include "../../ctl/include/ctl/ci/simuTRFEL.ci"

Ctlinterface::Ctlinterface()
{
}

//! Constructor.
/*!
  \param input           path to input file.
*/
Ctlinterface::Ctlinterface(const std::string& input)
{
  stochdrivert *stochd;
  stochd = new stochdrivert;
  
  Stt  = NULL;
  Kmat = NULL;
  Cmat = NULL;
  Jmat = NULL;
  Bmat = NULL;
  Lbt  = NULL;
  Lbat = NULL;
  Qbt   = NULL;
  Qbat  = NULL;
  Ltt   = NULL;
  Ltat  = NULL;
  Lqt   = NULL;
  Qqt   = NULL;
  Qqat  = NULL;
  Lqat  = NULL;
  Ltett = NULL;
  Lht   = NULL;
  Qht   = NULL;
  G2d   = NULL;

  static const char *argv[3];

  argv[0] = "trfel";
  argv[1] = input.c_str();
  argv[2] = NULL;
  
  //  program initiation and data reading
  trfel_init (2,argv,stochd);
}



//! Constructor.
/*!
  \param input           path to input file.
  \param params          parameters.
*/
Ctlinterface::Ctlinterface( const std::string& input, const std::vector<double>& params )
{
}


/**
  The function performs a cleanup of the TRFEL global variables
  and it closes opened files.

  Created 6.12.2012 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
Ctlinterface::~Ctlinterface()
{
  /*
  delete Tp;
  delete Gtt;
  delete Tt;
  delete Tm;
  delete Tc;
  delete Tb;
  delete Lsrst;
  delete Stt;
  delete Outdt;
  delete Kmat;
  delete Cmat;
  delete Jmat;
  delete Bmat;
  delete Lbt;
  delete Lbat;
  delete Qbt;
  delete Qbat;
  delete Ltt;
  delete Ltat;
  delete Lqt;
  delete Qqt;
  delete Qqat;
  delete Lqat;
  delete Ltett;
  delete Lht;
  delete Qht;
  */
  fclose(Outt);
}


//! Method to set parameters.
/*!
  \param params          parameters.
*/
void Ctlinterface::set_params( const std::vector<double>& params )
{
//  Stt->importvalues((double*) &params[0]);
}


    //! Method to solve/simulate the specified problem.
    /*!
      @param nstep - number of performed steps
      \return                0, if the solving was successful.
    */
int Ctlinterface::solve()
{
  long i,li,ret, j;
  double newtime,dt,end_time;
  np_glob_vec np_gv;
  long lcid = 0;

  //  nodes - integration points interpolation
  approximation();  
  nonstat_solver_init (lcid, np_gv);
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  //  end time
  end_time = Tp->timecont.endtime ();
  //  number of step
  li = i = np_gv.istep;
  j = 0;
  
  bool breakloop = false;
  // ***************************
  //  main iteration loop  ****
  // ***************************
  do{
    i++;
  
    newtime = Tp->time = Tp->timecont.newtime();
    //  computation of backward time step
    dt = Tp->timecont.actualbacktimeincr ();

    if(Tp->tprob == nonstationary_problem)
      // linear algorithm
      // perform one time step with linear solver
      ret = one_step_linear(lcid, newtime, dt, i, li, np_gv);
    
    if(Tp->tprob == nonlinear_nonstationary_problem)
      // non-linear algorithm
      // perform one time step with non-linear solver
      ret = one_step_nonlinear(lcid, newtime, dt, i, li, np_gv);
    
    /// adaptivity
      if (Tp->adaptivityflag  &&  Tp->time < end_time)
	if (i%2 == 0)
	  breakloop = Adat->run (2, true);
      j++;
      
  }while(Tp->time < end_time  &&  breakloop == false);

  return (Tp->time-end_time < 0.0);
}


    //! Method to get state/result/response (after method "solve").
    /*!
      \param state           state/result/response.
    */
void  Ctlinterface::get_state( std::vector<double>& state ) const
{
  long nt;
  nt = Tp->timecont.starttime() - Tp->timecont.endtime();
  nt /=Tp->timecont.initialtimeincr ();
 

  state.resize(Ndoft*(nt+1));
  double *statep= new double[Ndoft*(nt+1)];
  
  Stt->exportvalues((double*) statep);
  for(int i=0; i<Ndoft*(nt+1); i++)
    state[i] = statep[i];
}

struct connectDetail
{
  CTL_Constructor(1, (const std::string&), 1);
  CTL_Constructor(2, (const std::string&, const std::vector<double>&), 2);
};

void CTL_connect()
{
  ctl::connect<simuCI, Ctlinterface, connectDetail>();
}



    //! Method to get the residual.
    /*!
      \param params          parameters. these parameters come form outside, e.g. material parameters
      \param state           state. comes from outside, it contains e.g. displacements, temperatures, 
      \param niter           number of iterations for preconditioned residual (0=residual).
      \param residual        residual.
    */ 
void Ctlinterface::get_residual(const std::vector<double>& params,const std::vector<double>& state, 
                                const unsigned int niter, std::vector<double>& residual ) const
{
  long i, j, n=Ndoft, nt;
  double *aux, *lhs, *du, dt;
  aux = new double [n];
  du = new double [n]; 
  

  lhs = Lsrst->give_lhs(0);
  
  Stt->importvalues((double*) &params[0]);

  dt = Tp->timecont.initialtimeincr ();
  
  nt = Tp->timecont.starttime() - Tp->timecont.endtime();
  nt /= dt;

  for (i=0; i<nt+1; i++)
  {
    // set actual time in timecontr.

    for (j=0;j<n;j++){
      lhs[j]=state[n*i+j];
    }  
    
    subv(lhs, (double*) &state[n*(i+1)], du, n);
    residuum((double*) &residual[n*i], du, aux, dt, n, 0);
  }

  
  
  
//  Stt->changevalues ();
  
  delete [] du; 
  delete [] aux;
}

