#ifndef LINTETT_H
#define LINTETT_H

#include "genfile.h"

/**
   class lintett defines tetraheral element with linear approximation functions
   for transport problems
*/
class lintett
{
 public:
  lintett (void);
  ~lintett (void);
  void codnum (long *cn,long ri);
  double approx (vector &volcoord,vector &nodval);
  void intpointval (long eid);
  void intpointval_puc (long eid);
  void intpointgrad (long eid);
  void intpointother (long eid);

  void bf_matrix (matrix &n,vector &volcoord);
  void grad_matrix (matrix &gm,vector &b,vector &c,vector &d);
  void conductivity_matrix (long lcid,long eid,long ri,long ci,matrix &km);
  void l_matrix (long lcid,long eid,long ri,long ci,matrix &lm);
  void l_t_matrix (long lcid,long eid,long ri,long ci,matrix &lm);
  void capacity_matrix (long eid,long ri,long ci,matrix &cm);
  void quantity_source_vector (vector &sv,vector &nodval,long eid,long ri,long ci);
  void internal_fluxes (long lcid,long eid,vector &ifl);

  void res_conductivity_matrix (long eid,long lcid,matrix &km);
  void res_l_matrix (long eid,long lcid,matrix &lm);
  void res_l_t_matrix (long eid,long lcid,matrix &lm);
  void averd_matrix (long eid,matrix &lm);
  void averc_matrix (long eid,matrix &lm);
  double elem_volume (long eid);
  void res_capacity_matrix (long eid,matrix &cm);
  void res_convection_vector (vector &f,long lcid,long eid,long leid);
  void res_transmission_vector (vector &f,long lcid,long eid,long leid);
  void res_quantity_source_vector (vector &sv,vector &nodval,long lcid,long eid);
  void res_internal_fluxes (long eid,vector &elemif);
  double total_integral (long eid,vector &nodval);
  void res_boundary_flux (vector &f,long lcid,long eid,long leid);
  void nod_others (long lcid,long eid,long ri,long ci);
  

  void convection_vector (vector &v,long lcid,long eid,long leid,long ri,long ci);
  void transmission_matrix (long lcid,long eid,long ri,long ci,matrix &km);
  void transmission_vector (vector &v,long lcid,long eid,long leid,long ci);
  void boundary_flux (vector &v,long lcid,long eid,long leid,long ri,long ci);
  void surface_integral (long surf,vector &x,vector &y,vector &z,long intord,vector &gp1,vector &gp2,vector &w,
                         vector &coef,matrix &km);
  void transf_flux (long surf,vector &coeff,vector &list,vector &trc,vector &trr,long eid,long ri,long ci,long ipp,bocontypet *bc);
  void transf_coeff (long surf,vector &coeff,vector &list,long eid,long ri,long ci,long ipp,bocontypet *bc);
  void transf_val (long surf,vector &nodval,vector &list,vector &trc,vector &trr,long eid,long ri,long ci,long ipp,bocontypet *bc);
  void surfnodeval (long surf,vector &nodval,vector &list);
  
  void higher_to_lower_level (long eid,long *counter,double *buff);
  
  ///  number of transported matter
  long ntm;
  ///  total number of DOFs on the element
  long ndofe;
  ///  numbers of DOFs for particular problems
  long **dofe;
  ///  number of nodes on one element
  long nne;
  ///  number of edges
  long ned;
  ///  number of nodes on one edge
  long nned;
  ///  number of surfaces
  long nsurf;
  ///  number of nodes on one surface
  long nnsurf;
  ///  number of approximated functions
  long napfun;
  ///  problem dimension
  long ncomp;
  ///  number of integration points
  long **nip;
  ///  unknown ordering
  long **ordering;
  ///  orders of integration of conductivity matrices
  long **intordkm;
  ///  order of integration on boundaries(=surfaces)
  long intordb;
  ///  orders of integration of capacity matrices
  long **intordcm;
};

#endif
